@@ -410,10 +410,7 @@ int tevent_common_context_destructor(struct tevent_context *ev)
 
 	for (fd = ev->fd_events; fd; fd = fn) {
 		fn = fd->next;
-		tevent_trace_fd_callback(fd->event_ctx, fd, TEVENT_EVENT_TRACE_DETACH);
-		fd->wrapper = NULL;
-		fd->event_ctx = NULL;
-		DLIST_REMOVE(ev->fd_events, fd);
+		tevent_common_fd_disarm(fd);
 	}
 
 	ev->last_zero_timer = NULL;
@@ -518,7 +515,7 @@ void tevent_common_check_double_free(TALLOC_CTX *ptr, const char *reason)
 }
 
 /*
-  create a event_context structure for a specific implemementation.
+  create a event_context structure for a specific implementation.
   This must be the first events call, and all subsequent calls pass
   this event_context as the first element. Event handlers also
   receive this as their first argument.
@@ -547,6 +544,13 @@ struct tevent_context *tevent_context_init_ops(TALLOC_CTX *mem_ctx,
 	ev->ops = ops;
 	ev->additional_data = additional_data;
 
+	/*
+	 * have a default tick time of 30 seconds. This guarantees
+	 * that code that uses its own timeout checking will be
+	 * able to proceed eventually
+	 */
+	tevent_context_set_wait_timeout(ev, 30);
+
 	ret = ev->ops->context_init(ev);
 	if (ret != 0) {
 		talloc_free(ev);
@@ -585,6 +589,31 @@ struct tevent_context *tevent_context_init(TALLOC_CTX *mem_ctx)
 	return tevent_context_init_byname(mem_ctx, NULL);
 }
 
+uint32_t tevent_context_set_wait_timeout(struct tevent_context *ev,
+					 uint32_t secs)
+{
+	time_t ret = ev->wait_timeout.tv_sec;
+
+	/*
+	 * Cut to signed 32-bit time_t. This is the maximum that we
+	 * can reasonably expect to fit into time_t. Use that as magic
+	 * value for "wait forever". Nobody wants to wait 68 years....
+	 */
+	secs = MIN(secs, INT32_MAX);
+
+	/*
+	 * "secs == 0" and thus wait_timeout==tevent_timeval_zero()
+	 * might have a special meaning for custom tevent loop_once
+	 * backend functions. Make this an invalid value, our
+	 * internal poll and epoll backends deal with this properly,
+	 * see tevent_common_no_timeout()
+	 */
+	ev->wait_timeout = tevent_timeval_set((time_t)secs,
+					      (secs == 0) ?  INT32_MAX: 0);
+
+	return ret;
+}
+
 /*
   add a fd based event
   return NULL on failure (memory allocation error)
@@ -722,6 +751,14 @@ void _tevent_schedule_immediate(struct tevent_immediate *im,
 				    handler_name, location);
 }
 
+/*
+  reset an immediate event
+*/
+void tevent_reset_immediate(struct tevent_immediate *im)
+{
+	tevent_common_immediate_cancel(im);
+}
+
 /*
   add a signal event
 
@@ -824,7 +861,10 @@ int _tevent_loop_once(struct tevent_context *ev, const char *location)
 	tevent_trace_point_callback(ev, TEVENT_TRACE_AFTER_LOOP_ONCE);
 
 	/* New event (and request) will always start with call depth 0. */
-	tevent_thread_call_depth_set(0);
+	tevent_thread_call_depth_notify(TEVENT_CALL_FLOW_REQ_RESET,
+					NULL,
+					0,
+					__func__);
 
 	if (ev->nesting.level > 0) {
 		if (ev->nesting.hook_fn) {
